import argparse
from pathlib import Path

from pendulum import datetime

from ws_metrics.enums import FetchType, ReportEnum
from ws_metrics.queries.constructor import QueryConstructor, QueryString
from ws_metrics.queries.query_definitions import CAPABILITY_EXECUTIONS, DOWNLOAD_VOLUME, OPERATIONS
from ws_metrics.reporting.dispatch import DispatchReports


class LifeUniverseEverything:
    def __init__(self, connection, args: argparse.Namespace):
        self.conn = connection
        self.args = args
        self.constructor = QueryConstructor(self.conn)
        self.dispatch = DispatchReports()

    def get_total_cap_executions(self, capability_name: str) -> list:
        """
        Determine the total number of capability executions for a given capability that exist on this environment

        :param capability_name: the name of the capability to gather metrics for
        :return: the total number of capability executions
        """
        sql = QueryString(CAPABILITY_EXECUTIONS, False)
        params = {"capability_name": capability_name[0]}
        return self.constructor.construct_query(sql, params, FetchType.ONE)

    def get_total_executions_in_range(self, capability_name: str, beginning_date: datetime, end_date: datetime) -> list:
        """
        Determine the total number of executions of a specified capability

        :param capability_name: The name of the capability to gather metrics on
        :param beginning_date: The beginning of the date range of interest
        :param end_date: The end of the date range of interest
        :return: The number of capability executions that occurred in the specified date range
        """
        sql = QueryString(CAPABILITY_EXECUTIONS, True, 0)
        params = {"capability_name": capability_name, "beginning": beginning_date, "end": end_date}
        return self.constructor.construct_query(sql, params, FetchType.ONE)

    def get_request_data_volume(self, request_id: str) -> list:
        """
        Get the total downloaded data volume per capability request

        :param request_id: The capability request of interest
        :return: Data size in Kilobytes
        """
        sql = QueryString(DOWNLOAD_VOLUME, True, 0)
        params = {"request_id": request_id}
        return self.constructor.construct_query(sql, params, FetchType.ONE)

    def get_total_data_volume(self, beginning_date: datetime, end_date: datetime) -> list:
        """
        Determine the total data volume downloaded withing a specified date rge

        :param beginning_date: The beginning date of interest
        :param end_date: The ending date of interest
        :return: The total data size in Kilobytes
        """
        sql = QueryString(DOWNLOAD_VOLUME, True, 1)
        params = {"beginning": beginning_date, "end": end_date}
        return self.constructor.construct_query(sql, params, FetchType.ONE)

    def get_operations_report_info(self, beginning_date: datetime, end_date: datetime):
        """
        Run operations metrics report

        :return: report data
        """
        sql = QueryString(OPERATIONS, True)
        params = {"beginning": beginning_date, "end": end_date}
        return self.constructor.construct_query(sql, params, FetchType.ALL)

    # def get_operations_report_count(self, beginning_date: datetime, end_date: datetime) -> int:
    #     """
    #     Determine count of all datasets archived between two dates
    #
    #     :return: Number of archive datasets
    #     """
    #     sql = QueryString(OPERATIONS_COUNT, False)
    #     params = {"beginning": beginning_date, "end": end_date}
    #     return self.constructor.construct_query(sql, params, FetchType.ONE)

    def run_metrics(self, args: argparse.Namespace):
        """
        Run a specified metrics report. Output data to file or terminal

        :param args:
        :return:
        """
        report = None

        if args.capability:
            result = self.get_total_cap_executions(args.capability)
            report = self.dispatch.write_report(
                ReportEnum.CAPABILITY,
                {
                    "name": args.capability[0],
                    "total": str(result[0]),
                },
            )
        elif args.between:
            name = args.between[0]
            begin = args.between[1]
            end = args.between[2]
            result = self.get_total_executions_in_range(name, begin, end)
            report = self.dispatch.write_report(
                ReportEnum.CAPABILITY,
                {
                    "name": name,
                    "begin": begin,
                    "end": end,
                    "total": str(result[0]),
                },
            )
        elif args.requestdatasize:
            id = args.requestdatasize[0]
            result = self.get_request_data_volume(id)
            if result is not None:
                report = self.dispatch.write_report(
                    ReportEnum.DOWNLOAD,
                    {
                        "id": id,
                        "size": str(result[0]),
                    },
                )
            else:
                print("Request #" + args.requestdatasize[0] + " not found!")
                exit(1)
        elif args.datasize:
            begin = args.datasize[0]
            end = args.datasize[1]
            result = self.get_total_data_volume(begin, end)

            if result[0] is not None:
                report = self.dispatch.write_report(
                    ReportEnum.DOWNLOAD,
                    {
                        "begin": begin,
                        "end": end,
                        "size": str(result[0]),
                    },
                )
            else:
                print("Error: No download capability requests were found!")
                exit(1)
        elif args.ops:
            begin = args.ops[0]
            end = args.ops[1]
            result = self.get_operations_report_info(begin, end)
            report = self.dispatch.write_report(
                ReportEnum.OPERATIONS,
                {
                    "begin": begin,
                    "end": end,
                    "results": result,
                },
            )

        if args.report:
            if args.path:
                path = Path(args.path[0])
            else:
                path = None
            self.dispatch.dispatch_file_to_user(report, path)
        else:
            self.dispatch.dispatch_data_to_terminal(report)
